#-------------------------------------------------------------------------#
#
#         Extremiles: A new perspective on asymmetric least squares 
#         
#                     Updated: June 04, 2017 
#  
#-------------------------------------------------------------------------#

# NOTE: This code is not run time optimized. Problems have not appeared so far, 
#       but their absence cannot be guaranteed for.


#Content: 0) Packages required (line 104)
#
#         1) Quantile-based Hill's estimator (line 138)
#
#         2) Weissman quantile estimator (line 167)
#
#         3) Plug-in extremile estimator based on the asymptotic connection (line 190)
#
#         4) Ordinary ALS extremile estimator (line 216)
#
#         5) True extremiles of the Burr, frechet, Pareto and Student distributions (line 307)
#
#            5.1) True extremile of the Burr distribution (line 322)
#            5.2) True extremile of the Frechet distribution (line 358)
#            5.3) True extremile of the Pareto distribution (line 396) 
#            5.4) True extremile of the Student t(df) distribution (line 433) 
#
#         6)  Figures 1, 2, 3 in the main article (line 474) 
#      
#            6.1) Monte Carlo experiments (line 480)
#            6.2) Figure 1 (line 670)
#            6.3) Figure 2 (line 789)
#            6.4) Figure 3 (line 908)
#
#         7)  Figure 4 in the main article (line 1032)
#
#            7.1) left panel (line 1039)
#            7.2) right panel (line 1053)
#
#         8)  Application : Medical insurance data (line 1077)
#
#            8.1) Load SOA data (line 1085)
#            8.2) Quantile & exremile risk estimates (line 1100)
#            8.3) Figure 5 in the main article (line 1166)
#            8.4) Averaged estimates over the stable region [150,500] (line 1215)
#
#         9) Supplement B.1 (line 1237)
#
#            (9.1) Empirical quantile & Empirical extremile (line 1243)
#            (9.2) True extremile of the Gaussian distribution (line 1304)
#            (9.3) MC : Normal(0,1) (line 1347)
#            (9.4) True extremile of the exponentiel distribution (line 1453)
#            (9.5) MC : Exponentiel(1) (line 1497)
#            (9.6) True extremile of the Gamma distribution (line 1604)
#            (9.7) MC : Gamma(3,1) (line 1651)
#            (9.8) True extremile of the Uniform distribution (line 1756)
#            (9.9) MC : Uniform[0,1] (line 1800)
#            (9.10) True extremile of the Beta distribution (line 1905)
#            (9.11) MC : Beta(2,2) (line 1951)
#            (9.12) True extremile of the Chi-square distribution (line 2057)
#            (9.13) MC : Chi-square(3) (line 2101)
#            (9.14) True extremile of the Lognormal distribution (line 2208)
#            (9.15) MC : Lognormal(0,0.5) (line 2253)
#            (9.16) True extremile of the Weibull distribution (line 2373)
#            (9.17) MC :  Weibull(1,pi) (line 2417)
#            (9.18)  Figure 1 in Supplement B.1 (line 2522)
#
#         10)  Supplement B.2 (line 2670)
# 
#            (10.1) Empirical quantile (line 2676)
#            (10.2) Empirical extremile (line 2701)
#            (10.3) Empirical trimmed extremile (line 2742)
#            (10.4) True extremile of the Burr distribution (line 2769)
#            (10.5) Population trimmed extremile of the Burr distribution (line 2816)
#            (10.6) Monte Carlo comparison: Burr(4,1) (line 2868) 
#            (10.7) True extremile of the Pareto distribution (line 2991)
#            (10.8) Population trimmed extremile of the Pareto distribution (line 3034)
#            (10.9) Monte Carlo comparison: Pareto(4)  (line 3084)
#            (10.10) True extremile of the Student t(df) distribution (line 3197)
#            (10.11) Population trimmed extremile of the Student t(df) distribution (line 3242)
#            (10.12) Monte Carlo comparison : Student t(3)  (line 3294)
#            (10.13)  Figure 2 in Supplement B.2 (line 3421)
#
#         (11)  Supplement C  (line 3539)
#
#            (11.1) L-extremile  (line 3545)
#            (11.2) LM-extremile  (line 3585)
#            (11.3) M-extremile  (line 3618)
#            (11.4) PWM-extremile  (line 3645)
#            (11.5) Load data: Logreturns of the SP500 Index  (line 3716)
#            (11.6) Figure 3 in Supplement C  (line 3752)
#            (11.7) Calculations  (line 3766)
#            (11.8) Table 1 in Supplement C (line 3919)
#            (11.9) Figure 4 in Supplement C  (line 3939)
#
#///////////////////////////////////////////////////////////////////////
#///////////////////////////////////////////////////////////////////////


#__________________________________
#
#      (0) Packages required:
#
#__________________________________

##Clear the working space:         
  rm(list = ls())
# 
install.packages("expectreg")
require(expectreg)
#
install.packages("ggplot2")
require(ggplot2)
#
library(reshape2)
#
install.packages("VGAM")
require(VGAM)
#
install.packages("ggfortify")
library(ggfortify)
#
install.packages("zoo")
library(zoo)
#
install.packages("fGarch")
require(fGarch)
#
install.packages("actuar")
require(actuar)



#__________________________________
#
#  (1) Quantile-based Hill's estimator: 
#
#__________________________________

Hill_inverse  <- function(ytab,k)
{
	  #(Extremiles: by Daouia.A, Juanary 7, 2013)
      #inputs:
      #ytab = sample of nx1 obs
      #    k  = sample fraction
      #output: res = 1/(Hill estimator) = Moment Mn1      
n=length(ytab);
ysort=sort(ytab);   
        g1=ysort[n-k];
        som1=0;
        for (i in 0:(k-1))
        {
            gi=ysort[n-i];
            som1 = som1 + (log(gi)-log(g1));
        }
        res = som1/k;
        return(res);
}




#__________________________________
#
#   (2) Weissman quantile estimator: 
#
#__________________________________

Weissman_quantile <- function(ytab,pn,k)
{
       #inputs:
       #       ytab = sample of nx1 obs
       #        pn  = order < 1/n
       #         k  = sample fraction varying from "1" up to "n-1"
       #output: res = Weissman estimator of the extreme (1-pn)th quantile.
 
       n=length(ytab);
       ysort=sort(ytab);   
       gam = Hill_inverse(ytab,k);
       return(ysort[n-k]*(k/(n*pn))^(gam));
}




#__________________________________
#
#  (3) Plug-in extremile estimator based on the asymptotic connection:
#          
#__________________________________

Plugin_extremile <- function(ytab,pn,k)
{
       #inputs:
       #       ytab = sample of nx1 obs
       #        pn  = order < 1/n
       #         k  = sample fraction varying from "1" up to "n-1"       
       #output: res = Plug-in estimator of the extreme (1-pn)th extremile $\widehat\xi_{1-p_n}^{\star}$.

       q = Weissman_quantile(ytab,pn,k);
       s = Hill_inverse(ytab,k);
       ##s = s * (0<s) * (s<1);
       s = s * (s<1);
       Gs = gamma(1-s) * (log(2))^s;
      return(q * Gs);
}





#__________________________________
#
#  (4) Ordinary ALS extremile estimator: 
#          
#__________________________________

### internal functions
.sg <- function(gam)
{
	#input: gam = extremile order in (0,1)
  #output: power $s(gam)$ in the definition of the measure $K_{gam}$
  log(1/2)/log(1-gam)
}

# _________________________________

.Kg <- function(gam, t)
{
	#input: gam = extremile order in (0,1)
	#         t = real number in the support [0,1] of the $K_{gam}$
  #output: measure $K_{gam}(t)$ 
  ((1-(1-t)^(.sg(gam)))*(0<gam)*(gam<=0.5))+((t^(.sg(1-gam)))*(0.5<gam)*(gam<1))
}
# _________________________________

# .Jg <- function(gam, t)
# {
	# #input: gam = extremile order in (0,1)
	# #         t = real number in the support (0,1] of the density $J_{gam}=K'_{gam}$
  # #output: weight-generating function $J_{gam}(t)$ 
  # (((.sg(gam))*(1-t)^(.sg(gam)-1))*(0<gam)*(gam<=0.5))+(((.sg(1-gam))*t^(.sg(1-gam)-1))*(0.5<gam)*(gam<1))
# }

#An alternative : 

.Jg <- function(gam, t)
{
	#input: gam = extremile order in (0,1)
	#         t = real number in the support (0,1] of the density $J_{gam}=K'_{gam}$
  #output: weight-generating function $J_{gam}(t)$ 
  
  if ((0<gam) & (gam<=0.5)){res = (.sg(gam))*(1-t)^(.sg(gam)-1)}; 
  if ((0.5<gam) & (gam<1)) {res = (.sg(1-gam))*t^(.sg(1-gam)-1)};
  return(res)
}

### main functions

empirical_extremile <- function(ytab, gam)
{
  #inputs:
  #    ytab = sample of nx1 obs
  #    gam = extremile order in (0,1)
  #output: Ordinary $gam$th sample extremile $\widehat\xi_{gam}$.
  #__________________________________
 sytab=sort(ytab)
 n=length(ytab)
  compute<-function(gam, x)
   {res=0
    for (i in 1:n)
    {res = res + ((.Kg(gam,i/n)-.Kg(gam,(i-1)/n))*x[i])}
    return(res)
   }
 return(unlist(lapply(gam,compute,sytab)))
}

###

als_extremile <- function(ytab,probs)
{
       #inputs:
       #       ytab = sample of nx1 obs
       #        gam = order in (0,1)     
       #output: res = Ordinary $gam$th sample ALS estimator $\widetilde\xi_{gam}$.
       #$$ \widetilde{\xi}_{\gamma} = \frac{1}{n}  \sum_{i=1}^n  J_{gam}(i/n) * Ysort[i] $$
  #__________________________________
 sytab=sort(ytab)
 n=length(ytab)
    num=0;
    den=0;
    for (i in 1:n){
    num = num + (.Jg(probs,i/n)*sytab[i]);
    den = den + (.Jg(probs,i/n));    
    #print(.Jg(probs,i/n))
    }
    return(num/den)
}




#__________________________________
#
#  (5) True extremiles of the Burr, frechet, Pareto and Student distributions:  
#         
#__________________________________

# internal function
  .f1<-function(x, gam)
   {cste<-ifelse(gam<=0.5, 1-gam, gam)
     if(gam<=0.5)
      {(log(1/2)/log(cste))*(1-x)^((log(1/2)/log(cste))-1)}
     else
      {(log(1/2)/log(cste))*x^((log(1/2)/log(cste))-1)}
    }

#__________________________________
#
#  (5.1) True extremile of the Burr distribution:  
#         
#__________________________________

xburr <- function(gam, shape1, shape2, rate = 1)
{
  #inputs:
  #        gam   = extremile order in (0,1)
  #        shape1, shape2, rate = parameters. Must be strictly positive (see Actuar: Burr distribution). 
  #output: theoretical (gam)th extremile $\xi_{gam}$   
   #__________________________________     
      
      compute <- function(gam, shape1, shape2, rate)
      {
       f <- function(x, shape1, shape2, rate, gam)
        {
         qburr(x, shape1, shape2, rate)*.f1(x,gam)
        }
       integrate(f, lower = 0, upper = 1, shape1=shape1, shape2=shape2, rate=rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value
      }

      gam[gam<=0 | gam>=1] <- NA
      res<-rep(NA,length(gam))
      res[!is.na(gam)]<-unlist(lapply(na.omit(gam), compute, shape1, shape2, rate))
      return(res)
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xburr(gam, 4,1,1)
     # alpha = pburr(xburr(gam, 4,1,1), 4,1,1);  #\alpha(\gamma) 
     # qburr(alpha, 4,1,1)


#__________________________________
#
#  (5.2) True extremile of the Frechet distribution:  
#         
#__________________________________
 
xfrechet <- function(gam, loc = 0, scale = 1, shape)
{
  #inputs:
  #        gam   = extremile order in (0,1)
  #        loc, scale, shape = the location parameter a, scale parameter b, and shape parameter s. 
  #output: gives the theoretical (gam)th extremile $\xi_{gam}$   
  #__________________________________   
               
      compute <- function(gam, loc, scale, shape)
      {
       f <- function(x, loc, scale, shape, gam)
        {
         qfrechet(x, loc, scale, shape)*.f1(x,gam)
        }
       integrate(f, lower = 0, upper = 1, loc=loc, scale=scale, shape=shape, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value
      }

      gam[gam<=0 | gam>=1] <- NA
      res<-rep(NA,length(gam))
      res[!is.na(gam)]<-unlist(lapply(na.omit(gam), compute, loc, scale, shape))
      return(res)
}

#     ##Example [check that: Quantile(alpha) = Extremile(gam)]
#     shape=5;
#     gam = 0.99; 
#     xfrechet(gam=gam, loc = 0, scale = 1,shape=shape)
#     alpha = pfrechet(xfrechet(gam, loc = 0, scale = 1,shape=shape), shape=shape);  #\alpha(\gamma)
#     qfrechet(alpha, shape=shape)



#__________________________________
#
#  (5.3) True extremile of the Pareto distribution:  
#                  
#__________________________________
 
xpareto <- function(gam, shape, scale)
{
  #inputs:
  #        gam   = extremile order in (0,1)
  #        shape, scale = parameters. Must be strictly positive. 
  #output: theoretical (gam)th extremile $\xi_{gam}$   
  #__________________________________ 
                 
      compute <- function(gam, shape, scale)
      {
       f <- function(x, shape, scale, gam)
        {
         qpareto(x, shape, scale)*.f1(x,gam)
        }
       integrate(f, lower = 0, upper = 1, shape=shape, scale=scale, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value
      }

      gam[gam<=0 | gam>=1] <- NA
      res<-rep(NA,length(gam))
      res[!is.na(gam)]<-unlist(lapply(na.omit(gam), compute, shape, scale))
      return(res)
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xpareto(gam, 2, 2)
     # alpha = ppareto(xpareto(gam, 2, 2), 2, 2);  #\alpha(\gamma)
     # qpareto(alpha, 2, 2)
     
     

#__________________________________
#
#  (5.4) True extremile of the Student t(df) distribution:  
#          
#__________________________________
 
xt <- function(gam, df, ncp=0)
{
  #inputs:
  #        gam = extremile order in (0,1]
  #        df  = Student's degree of freedom
  #output: theoretical (gam)th extremile $\xi_{gam}$   
  #__________________________________
                         
      compute <- function(gam, df, ncp)
      {
       f <- function(x, df, ncp, gam)
        {
         qt(x, df, ncp)*.f1(x,gam)
        }
       integrate(f, lower = 0, upper = 1, df=df, ncp=ncp, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value
      }

      gam[gam<=0 | gam>=1] <- NA
      res<-rep(NA,length(gam))
      res[!is.na(gam)]<-unlist(lapply(na.omit(gam), compute, df, ncp))
      return(res)
}

     ##Example [check that: Quantile(alpha) = Extremile(gam)]
     #df=3;
     #gam = 0.99; 
     #xt(gam, df)
     #alpha = pt(xt(gam, df), df);  #\alpha(\gamma) 
     #qt(alpha, df)






#______________________________________________________________________________________________________
#
#   (6)  Figures 1, 2, 3
#
#______________________________________________________________________________________________________

 
#---------------------------------------------------------------------------------------------------
#    (6.1)  Monte Carlo experiments:                             
#---------------------------------------------------------------------------------------------------
#sample size:
      n = 1000;

#tail index:
      ro = 4  #3  #2.5 
 
#order "pn < 1/n" [extremile level gamma'_n = 1-pn]:
      pn = 1/n;                                                      
 
#grid of values of "k":
kl = floor(log(n^0.9))+1;  
ku = floor(n/log(n^0.9)); 
kk = kl:ku; 
nk = length(kk);
 
#Nb of simulated samples:
B = 15000;
                 #---------------------------------------------------#
                 #        True extremile $\xi_{1-pn}$:
                 #---------------------------------------------------#
                                                   
                 #Frechet:
                 tVaR = xfrechet(1-pn, loc=0, scale=1, shape=ro) * matrix(1,B,nk);  
                 
                                 
                 # #Pareto:
                 # tVaR = xpareto(1-pn, shape=ro, scale=1) * matrix(1,B,nk);                  
                 

                 # #Student:
                 # tVaR = xt(1-pn, df=ro, ncp=0) * matrix(1,B,nk); 
                                                   
                 #---------------------------------------------------#
     
#Initialization of the VaR estimators (B simulated samples & nk estimates for k=kl:ku):
       xp = matrix(0,B,nk);
   xp_als = matrix(0,B,nk);
   
#Monte-Carlo trials:
for (b in 1:B)
{	
                 #---------------------------------------------------#
                 #            Simulated data:
                 #---------------------------------------------------#

                 #Frechet:
                 ytab = rfrechet(n, location=0, scale=1, shape=ro);
                 
                 
                 # #Pareto:
                 # ytab = rpareto(n, scale=1, shape=ro);
                  
                  
                 # #Student:
                 # ytab = rt(n, ro); 


                 #---------------------------------------------------#
                 #          Calculation of the estimates:
                 #---------------------------------------------------#
                  
                 for (i in 1:nk)
                 {        
            	   k=kk[i];
               	  #Hill's EVI:
            	   gam = Hill_inverse(ytab,k);
                  #Indirect extremile-VaR $\widehat\xi_{1-p_n}^{\star}$:
                   xp[b,i] = Plugin_extremile(ytab,pn,k);
                  #Direct extremile-VaR $\widetilde\xi_{1-p_n}^{\star}$:
#                   xp_als[b,i] = (n*pn/k)^(-gam) * empirical_extremile(ytab,1-(k/n));
                   xp_als[b,i] = (n*pn/k)^(-gam) * als_extremile(ytab,probs=1-(k/n));                                                                
                 }
}
                 #---------------------------------------------------#
                 #          MSE & Bias results:
                 #---------------------------------------------------#

        xp_MSE = colMeans((tVaR - xp)^2,na.rm=TRUE); #Indirect (plug-in) Xtremile estimator
        xp_als_MSE = colMeans((tVaR - xp_als)^2);    #Direct (LAWS) Xtremile estimator
        
        xp_Bias = colMeans(xp - tVaR,na.rm=TRUE);      
        xp_als_Bias = colMeans(xp_als - tVaR);
       
               
        ###########################################
        #          Root MSE                       #
        ###########################################        
        # #Root MSE:
        # kk = kk[2:154];
        # xp_als_MSE = sqrt(xp_als_MSE[2:154]);
        # xp_MSE = sqrt(xp_MSE[2:154]);  
        # xp_als_Bias = xp_als_Bias[2:154];
        # xp_Bias = xp_Bias[2:154];

        # #Root MSE:
        # xp_als_MSE = sqrt(xp_als_MSE);
        # xp_MSE = sqrt(xp_MSE);  
                    
       
        ###########################################
        #          Normalized Root-MSE            #
        ###########################################        
        # xp_als_NMSE = xp_als_MSE/abs(tVaR[1,1]); 
        # xp_NMSE = xp_MSE/abs(tVaR[1,1]);


        ###########################################
        #          Normalized MSE                 #
        ###########################################        
        xp_als_NMSE = xp_als_MSE/(tVaR[1,1])^2; 
        xp_NMSE = xp_MSE/(tVaR[1,1])^2;
        
                   
        ###########################################
        #             Normalized BIAS             #
        ###########################################        
        xp_NBias = xp_Bias/tVaR[1,1];
        xp_als_NBias = xp_als_Bias/tVaR[1,1];
        
        
        # u=3;
        # kk = kk[u:154];
        # xp_als_MSE = xp_als_MSE[u:154];
        # xp_MSE = xp_MSE[u:154];  
        # xp_als_Bias = xp_als_Bias[u:154];
        # xp_Bias = xp_Bias[u:154];
        


  # dev.new()
# df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    # df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

# dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
# dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
# #scale_y_continuous("MSE", limit = c(0,.15)) + # rho=2.5
# scale_y_continuous("MSE") +
# scale_x_continuous("k")+
# #labs(title = "Fréchet")+ 
# theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
# panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))


  # dev.new()
# df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    # df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

# dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
# dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
# scale_y_continuous("BIAS") +
# scale_x_continuous("k")+
# #labs(title = "Fréchet")+ 
# theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
# panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))


#---------------------------------------------------------------------------------------------------
#Save data:
#---------------------------------------------------------------------------------------------------

##### rho = 3 #####
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student.RData")


##### rho = 4 #####
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet4.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto4.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student4.RData")


##### rho = 2.5 #####
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet2v5.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto2v5.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student2v5.RData")




#---------------------------------------------------------------------------------------------------
#
#    (6.2)  Figure 1 :   MSE_Bias4.pdf   
#                            
#---------------------------------------------------------------------------------------------------

  dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,3)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet4.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet_4.RData")
    
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
#scale_y_continuous("MSE") +
scale_y_continuous("MSE", limit = c(0.016,0.13)) + # rho=2.5
#scale_y_continuous("MSE", limit = c(0.19,0.53)) + # rho=3
scale_x_continuous("k")+ # rho=4
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))
   
  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto4.RData")
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto_4.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.1,2.75)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,2))

    
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student4.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student_4.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.1,2.75)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,3))

   
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet4.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet_4.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1))


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto4.RData")
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto_4.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,2))

  
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student4.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student_4.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,3))




#---------------------------------------------------------------------------------------------------
#
#    (6.3)  Figure 2 :   MSE_Bias.pdf  
#                            
#---------------------------------------------------------------------------------------------------

  dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,3)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet_3.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
#scale_y_continuous("MSE") +
scale_y_continuous("MSE", limit = c(0.03,0.4)) + # rho=2.5
#scale_y_continuous("MSE", limit = c(0.19,0.53)) + # rho=3
scale_x_continuous("k")+ # rho=4
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))
   
  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto.RData")
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto_3.RData")
    
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.25,1.5)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,2))

    
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student_3.RData")  
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.15,3)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,3))

   
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet_3.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1))


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto.RData")
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto_3.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,2))

  
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student_3.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,3))




#---------------------------------------------------------------------------------------------------
#
#  (6.4)  Figure 3 :   MSE_Bias2p5.pdf   
#                            
#---------------------------------------------------------------------------------------------------

  dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,3)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet2v5.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet_2.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
#scale_y_continuous("MSE") +
scale_y_continuous("MSE", limit = c(0.06,0.61)) + # rho=2.5
#scale_y_continuous("MSE", limit = c(0.19,0.53)) + # rho=3
scale_x_continuous("k")+ # rho=4
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))
   
  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto2v5.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.4,1.5)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,2))

    
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student2v5.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student_2.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0,4)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,3))

   
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet2v5.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//frechet_2.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1))


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//pareto2v5.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,2))

  
#  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student2v5.RData")
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//student_2.RData")
  
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,3))










#————————————————————————————————————————————————————————————————————————————————
#
#
#   (7)  Figure 4
#
#
#————————————————————————————————————————————————————————————————————————————————

  dev.new()
#————————————————
#  (7.1) Figure 4 (left panel): R_losses_hist.pdf
#————————————————
#first step:
losses<-read.table("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes/hurricane.m", sep=",",dec=".", skip=1)
#second step:
losses<-read.table("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes/hurricane.m", sep=",",dec=".", skip=1,nrows=35)
Losses <- losses$V1
m <- ggplot(losses, aes(x=Losses))
#m + geom_histogram(colour = "darkblue", fill = "lightblue", binwidth = 1.5*10^5) + geom_rug(sides="b",colour = "red")
m + geom_histogram(colour = "darkgreen", fill = "white", binwidth = 1.5*10^5) + geom_rug(sides="b",colour = "red")


 dev.new()
#————————————————
# (7.2) Figure 4 (right panel) :  R_gamma_risks.pdf
#————————————————
don<-read.table("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes/hurricanes_results", sep=",",dec=".")
Extremile = don$V2;
Quantile = don$V4;
Tail_mean = don$V5;
Expectile = don$V3;
gamma=don$V1;
 df <- data.frame(gamma, Quantile, Expectile, Extremile, Tail_mean)
 names(df) <- c("gamma", "Quantile", "Expectile", "Extremile", "Tail mean")
 ecm <- melt(df, id = "gamma")
 dd <- qplot(gamma, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
 dd+scale_color_manual(values = c("Extremile" = '#FF0000','Quantile' = '#3399ff', "Tail mean" = 'orange', "Expectile" =  '#32CD32'))  + scale_y_continuous(expression(Risk (tau))) + scale_x_continuous(expression(tau))
 
 
 
 
 
 

#______________________________________________________________________________________________________
#
#
#
#    (8)  Application : Medical insurance data
#
#
#
#______________________________________________________________________________________________________
#__________________________________
#
#
#  (8.1) Load SOA data:
#
#
#__________________________________
#first step:
don<-read.table("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes/soa.m", sep=",",dec=".", skip=1)
#second step:
don<-read.table("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes/soa.m", sep=",",dec=".", skip=1,nrows=75789)
Claims <- don$V1
ytab <- Claims


#————————————————---------------
#
#
#  (8.2) Quantile & exremile risk estimates
#
#
#————————————————---------------
##Sample size:
 n = length(ytab);
 Yn = max(ytab);
 ysort=sort(ytab);   

##order "pn < 1/n":
 pn = 1/100000;

##grid of values of "k":
 kl = 3; ku = 700;
 kk = kl:ku; 
 nk = length(kk);

##Initialization of the risk estimates:
gam =  cbind(rep(0, nk));      #Hill's EVI
rho =  cbind(rep(0, nk));      #Inverse of Hill's EVI

qu  = cbind(rep(0, nk));       #quantile-VaR

xp = cbind(rep(0, nk));        #Indirect expectile-VaR
xp_als = cbind(rep(0, nk));    #Direct expectile-VaR

xt = cbind(rep(0, nk));        #Indirect extremile-VaR
xt_als = cbind(rep(0, nk));    #Direct extremile-VaR

line_max = cbind(rep(0, nk));    #Sample maximum

##Calculation of the risk estimates:
for (i in 1:nk)
        { 
        	k=kk[i];
        	#Sample maximum:
        	line_max[i] = Yn; 
        	#Hill's EVI:
        	gam[i] = Hill_inverse(ytab,kk[i]);
        	     #Inverse of Hill's EVI:
        	     rho[i]= 1/gam[i];
              
              #quantile-VaR $\hat{q}_{1-p_n}^{\star}$: 
              qu[i]=Weissman_quantile(ytab,pn,k);                            
              
              #Estimate $\widehat\tau_n'(\alpha_n)$ of the extreme expectile level $\tau_n'(\alpha_n)$:
              htau = 1 - ((pn) * (gam[i]/(1-gam[i])));              
              
              #Indirect expectile-VaR $\hat{e}_{\widehat\tau_n'(1-pn)}^{\star}$:
              xp[i] = (rho[i] - 1)^(-gam[i]) * Weissman_quantile(ytab,(1-htau),k);
                                 
              #Direct expectile-VaR $\widetilde{e}_{\widehat\tau_n'(1-pn)}^{\star}$:
              xp_als[i] = (n*(1-htau)/k)^(-gam[i]) * expectile(ytab,probs=1-(k/n)); #here we use the "expectreg" package
              
              #Indirect extremile-VaR $\widehat\xi_{1-p_n}^{\star}$:
              xt[i] = Plugin_extremile(ytab,pn,k);
              
              #Direct extremile-VaR $\widetilde\xi_{1-p_n}^{\star}$:
              #xt_als[i] = (n*pn/k)^(-gam[i]) * empirical_extremile(ytab,1-(k/n));
              xt_als[i] = (n*pn/k)^(-gam[i]) * als_extremile(ytab,probs=1-(k/n));                          
           }
         

#_______________________________________________________________
#
#
#   (8.3)  Figure 5 :   soa_risks.pdf
#
#
#_______________________________________________________________   
df_xt <-data.frame(xxc = kk, yyc = xt, gam)         # rainbow (solid)
df_xt_als <-data.frame(xxc = kk, yyc = xt_als, gam) # yellow (dotted)    
             
df_max <-data.frame(xxc = kk, yyc = line_max)       # pink (solid)                   

df_qu <-data.frame(xxc=kk, yyc = qu, gam)           # black (solid)  
df_xp_als <-data.frame(xxc = kk, yyc = xp_als, gam) # gray (dashed)
                   
   dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

m <- ggplot(don, aes(x=log(Claims)))

print(m + geom_histogram(aes(fill = ..count..)) +
 scale_fill_gradient("Count", low = "red", high = "yellow")+ geom_rug(sides="b",colour = "lightblue")+
labs(title = "(a) Histogram")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
 vp=define_region(1,1:2))   
       
dd <- ggplot(data=df_xt, aes(xxc, yyc,col=gam)) + geom_line(size = 1.5) 
print(dd+geom_line(size = 1.5,data = df_max, colour = "pink", linetype=1)+
scale_colour_gradientn(name=expression(hat(gamma)[H]), colours=rainbow(7)) + 
scale_y_continuous(name = "Risk  Measure", limit = c(3.5*10^6,6.5*10^6))+
scale_x_continuous("Sample  fraction  k", limit = c(3, 700))+
geom_line(size = 1,data = df_xt_als, colour = "black", linetype=3)+
geom_line(size = 1,data = df_qu, colour = "black", linetype=1)+
geom_line(size = 1,data = df_xp_als, colour = "gray", linetype=2)+
labs(title = "(b) Plots")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")) ,
  vp=define_region(2,1:2))



#_______________________________________________________________
#
#
#   (8.4)  Averaged estimates over the stable region [150,500]:
#
#
#_______________________________________________________________
ind_l = which(kk==150);
ind_u = which(kk==500);

#Averaged estimate:
c( min(qu[ind_l:ind_u]) ,  max(qu[ind_l:ind_u])  ,  mean(qu[ind_l:ind_u]) ) 

c( min(xp_als[ind_l:ind_u])  , max(xp_als[ind_l:ind_u]) , mean(xp_als[ind_l:ind_u]) )

c( min(xt[ind_l:ind_u]) , max(xt[ind_l:ind_u]) , mean(xt[ind_l:ind_u]) ) 

c( min(xt_als[ind_l:ind_u]) , max(xt_als[ind_l:ind_u]) , mean(xt_als[ind_l:ind_u]) )



 
#_______________________________________________________________
#
#
#   (9)  Supplement B.1:
#
#
#_______________________________________________________________
#__________________________________
#
#  (9.1) Empirical quantile & Empirical extremile: 
#
#__________________________________

empirical_quantile  <- function(ytab,gam)
{
	  #(Extremiles: by Daouia.A, May 4, 2015)
      #inputs:
      #ytab = sample of nx1 obs
      # gam = quantile order in (0,1)
      #output: $gam$th sample quantile      
      
ysort=sort(ytab);
n=length(ytab);
       alphan=gam*n;
       if (floor(alphan) == alphan)
           {res=ysort[alphan];}
       else 
           {res=ysort[floor(alphan)+1];}
       return(res);
}



sg <- function(gam)
{
	#input: gam = extremile order in (0,1)
      #output: power $s(gam)$ in the definition of the measure $K_{gam}$ [see daouia & Gijbels (Extremiles)]
      
return(log(1/2)/log(1-gam));
}

Kg <- function(gam,t)
{
	#input: gam = extremile order in (0,1)
	#           t = real number in the support [0,1] of the $K_{gam}$
      #output: measure $K_{gam}(t)$ [see daouia & Gijbels (Extremiles)]
res=((1-(1-t)^(sg(gam)))*(0<gam)*(gam<=0.5))+((t^(sg(1-gam)))*(0.5<gam)*(gam<1));     
return(res);
}


empirical_extremile <- function(ytab,gam)
{
	  #(Extremiles: by Daouia.A, May 4, 2015)
      #inputs:
      #ytab = sample of nx1 obs
      # gam = extremile order in (0,1)
      #output: $gam$th sample extremile          

sytab=sort(ytab);
res=0;
n=length(ytab);
for (i in 1:n)
    {res = res + ((Kg(gam,i/n)-Kg(gam,(i-1)/n))*sytab[i]);}
return(res);
}


#__________________________________
#
#  (9.2) True extremile of the Gaussian distribution:  
#             N(mean,sd)
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,mean,sd,gam) {qnorm(x, mean, sd)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(mean,sd,gam) {integrate(left, lower = 0, upper = 1, mean=mean, sd=sd, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(0,1,0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,mean,sd,gam) {qnorm(x, mean, sd)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(mean,sd,gam) {integrate(right, lower = 0, upper = 1, mean=mean, sd=sd, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(0,1,0.5)

 
xnorm <- function(gam, mean, sd)
{
       #inputs:
       #        gam = extremile order in (0,1)
       #        (mean,sd)  = mean & standard deviation 
       #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft(mean,sd,gam);}
       else {res = xiright(mean,sd,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xnorm(gam, mean=0, sd=1)
     # alpha = pnorm(xnorm(gam, mean=0, sd=1), mean=0, sd=1);  #\alpha(\gamma) = ...
     # qnorm(alpha, mean=0, sd=1)



#_______________________________________________________________
#
#   (9.3) MC : Normal(0,1) :
#
#_______________________________________________________________
 #values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	#--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
   	alpha = pnorm(xnorm(gam, mean=0, sd=1), mean=0, sd=1);       #Normal(mean,sd)
	
	#True extremile(gamma) = true quantile(alpha):
	tVaR = xnorm(gam, mean=0, sd=1) * matrix(1,B,1);  #Normal(mean,sd)
	
	#------------------------------------------------------------------------------------------------------


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rnorm(n, mean=0, sd=1);       #Normal(mean,sd)
	#------------------------------------------------------------------------------------------------------
 
                  
                 #computation of the estimates:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_extremile(ytab,gam);                 
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 

# #-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//nromal.RData")
# ##


 
#__________________________________
#
#  (9.4) True extremile of the exponentiel distribution:  
#             Exp(1)
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,rate,gam) {qexp(x, rate)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(rate,gam) {integrate(left, lower = 0, upper = 1, rate=rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(1,0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,rate,gam) {qexp(x, rate)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(rate,gam) {integrate(right, lower = 0, upper = 1, rate=rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(1,0.5)

 
xexp <- function(gam, rate)
{
       #(Extremiles: by Daouia.A, May 5, 2015)
       #inputs:
       #        gam = extremile order in (0,1)
       #        rate  = 1/mean
       #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft(rate,gam);}
       else {res = xiright(rate,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xexp(gam, rate = 1)
     # alpha = pexp(xexp(gam, rate = 1), rate = 1);  #\alpha(\gamma) = ...
     # qexp(alpha, rate = 1)



#_______________________________________________________________
#
#   (9.5) MC : Exponentiel(1) :
#
#_______________________________________________________________

#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	#--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
    alpha = pexp(xexp(gam, rate = 1), rate = 1);
	
	#True extremile(gamma) = true quantile(alpha):
	tVaR = xexp(gam, rate = 1) * matrix(1,B,1);  
	
	#------------------------------------------------------------------------------------------------------


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rexp(n, rate = 1);      
	#------------------------------------------------------------------------------------------------------
 
                  
                 #computation of the estimates:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_extremile(ytab,gam);                 
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 

# #-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//exponentiel.RData")
# ##



#__________________________________
#
#  (9.6) True extremile of the Gamma distribution:  
#             Gamma(shape, rate = 1, scale = 1/rate)
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,shape, rate = 1, scale = 1/rate,gam) {qgamma(x, shape, rate = 1, scale = 1/rate)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(shape, rate = 1, scale = 1/rate,gam) {integrate(left, lower = 0, upper = 1, shape=shape, rate = 1, scale = 1/rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(shape=3, rate = 1, scale = 1/rate, gam = 0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,shape, rate = 1, scale = 1/rate,gam) {qgamma(x, shape, rate = 1, scale = 1/rate)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(shape, rate = 1, scale = 1/rate,gam) {integrate(right, lower = 0, upper = 1, shape=shape, rate = 1, scale = 1/rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(shape=3, rate = 1, scale = 1/rate, gam = 0.5)

 
xgamma <- function(gam, shape, rate = 1, scale = 1/rate)
{
       #(Extremiles: by Daouia.A, May 5, 2015)
       #inputs:
       #        gam = extremile order in (0,1)       
            #              rate =   an alternative way to specify the scale.
            #shape, scale  = 
            # shape and scale parameters. Must be positive, scale strictly.
            #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft(shape, rate = 1, scale = 1/rate,gam);}
       else {res = xiright(shape, rate = 1, scale = 1/rate,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xgamma(gam, shape=3, rate = 1, scale = 1/rate)
     # alpha = pgamma(xgamma(gam, shape=3, rate = 1, scale = 1/rate), shape=3);  
     # #\alpha(\gamma) = ...
     # qgamma(alpha, shape=3)



#_______________________________________________________________
#
#   (9.7) MC : Gamma(3,1) :
#
#_______________________________________________________________
#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	#--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
    alpha = pgamma(xgamma(gam, shape=3, rate = 1, scale = 1/rate), shape=3);
	
	#True extremile(gamma) = true quantile(alpha):
	tVaR = xgamma(gam, shape=3, rate = 1, scale = 1/rate) * matrix(1,B,1);  
	
	#------------------------------------------------------------------------------------------------------


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rgamma(n, shape=3);      
	#------------------------------------------------------------------------------------------------------
 
                  
                 #computation of the estimates:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_extremile(ytab,gam);                 
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 

# #-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//gamma.RData")
# ##


#__________________________________
#
#  (9.8) True extremile of the Uniform distribution:  
#             Unif(0,1)
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,min,max,gam) {qunif(x, min, max)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(min,max,gam) {integrate(left, lower = 0, upper = 1, min=min, max=max, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(0,1,0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,min,max,gam) {qunif(x, min, max)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(min,max,gam) {integrate(right, lower = 0, upper = 1, min=min, max=max, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(0,1,0.5)

 
xunif <- function(gam, min, max)
{
       #(Extremiles: by Daouia.A, May 5, 2015)
       #inputs:
       #        gam = extremile order in (0,1)
       #        (min,max)  = bounds of the distribution support
       #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft(min,max,gam);}
       else {res = xiright(min,max,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xunif(gam, min=0, max=1)
     # alpha = punif(xunif(gam, min=0, max=1), min=0, max=1);  #\alpha(\gamma) = ...
     # qunif(alpha, min=0, max=1)



#_______________________________________________________________
#
#   (9.9) MC : Uniform[0,1] :
#
#_______________________________________________________________
#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	#--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
    alpha = punif(xunif(gam, min=0, max=1), min=0, max=1);
	
	#True extremile(gamma) = true quantile(alpha):
	tVaR = xunif(gam, min=0, max=1) * matrix(1,B,1);  
	
	#------------------------------------------------------------------------------------------------------


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = runif(n, min=0, max=1);      
	#------------------------------------------------------------------------------------------------------
 
                  
                 #computation of the estimates:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_extremile(ytab,gam);                 
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 

# #-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//uniform.RData")
# ##


#__________________________________
#
#  (9.10) True extremile of the Beta distribution:  
#             Beta(shape1, shape2, ncp = 0)
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,shape1, shape2, ncp = 0,gam) {qbeta(x, shape1, shape2, ncp = 0)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(shape1, shape2, ncp = 0,gam) {integrate(left, lower = 0, upper = 1, shape1=shape1, shape2=shape2, ncp = ncp, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(shape1=2, shape2=2, ncp = 0, gam = 0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,shape1, shape2, ncp = 0,gam) {qbeta(x, shape1, shape2, ncp = 0)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(shape1, shape2, ncp = 0,gam) {integrate(right, lower = 0, upper = 1, shape1=shape1, shape2=shape2, ncp = ncp, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(shape1=2, shape2=2, ncp = 0, gam = 0.5)

 
xbeta <- function(gam, shape1, shape2, ncp = 0)
{
       #(Extremiles: by Daouia.A, June 1, 2015)
       #inputs:
       #        gam = extremile order in (0,1)       
       # shape1, shape2 = non-negative parameters of the Beta distribution
            #  ncp = non-centrality parameter
            # output: gives the theoretical (gam)th extremile $\xi_{gam}$
            
           if (gam <= 0.5){res = xileft(shape1, shape2, ncp = 0,gam);}
       else {res = xiright(shape1, shape2, ncp = 0,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xbeta(gam, shape1=2, shape2=2, ncp = 0)
     # alpha = pbeta(xbeta(gam, shape1=2, shape2=2, ncp = 0), shape1=2, shape2=2);  
     # #\alpha(\gamma) = ...
     # qbeta(alpha, shape1=2, shape2=2)



#_______________________________________________________________
#
#   (9.11)  MC : Beta(2,2) :
#
#_______________________________________________________________
#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	#--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
    alpha = pbeta(xbeta(gam, shape1=2, shape2=2, ncp = 0), shape1=2, shape2=2);
	
	#True extremile(gamma) = true quantile(alpha):
	tVaR = xbeta(gam, shape1=2, shape2=2, ncp = 0) * matrix(1,B,1);  
	
	#------------------------------------------------------------------------------------------------------


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rbeta(n, shape1=2, shape2=2);      
	#------------------------------------------------------------------------------------------------------
 
                  
                 #computation of the estimates:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_extremile(ytab,gam);                 
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 


# #-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//beta.RData")
# ##


#__________________________________
#
#  (9.12) True extremile of the Chi-square distribution:  
#             Chi-square(df)
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,df,gam) {qchisq(x, df)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(df,gam) {integrate(left, lower = 0, upper = 1, df=df, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(3,0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,df,gam) {qchisq(x, df)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(df,gam) {integrate(right, lower = 0, upper = 1, df=df, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(3,0.5)

 
xchisq <- function(gam, df)
{
       #(Extremiles: by Daouia.A, May 5, 2015)
       #inputs:
       #        gam = extremile order in (0,1)
       #        df  = degrees of freedom (non-negative, but can be non-integer).
       #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft(df,gam);}
       else {res = xiright(df,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xchisq(gam, df = 3)
     # alpha = pchisq(xchisq(gam, df = 3), df = 3);  #\alpha(\gamma) = ...
     # qchisq(alpha, df = 3)



#_______________________________________________________________
#
#   (9.13) MC : Chi-square(3) :
#
#_______________________________________________________________
#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	#--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
    alpha = pchisq(xchisq(gam, df = 3), df = 3);
	
	#True extremile(gamma) = true quantile(alpha):
	tVaR = xchisq(gam, df = 3) * matrix(1,B,1);  
	
	#------------------------------------------------------------------------------------------------------


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rchisq(n, df = 3);      
	#------------------------------------------------------------------------------------------------------
 
                  
                 #computation of the estimates:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_extremile(ytab,gam);                 
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,];   

# #-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//Chisquare.RData")
# ##




#__________________________________
#
#  (9.14) True extremile of the Lognormal distribution:  
#            
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,meanlog,sdlog,gam) {qlnorm(x, meanlog,sdlog)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(meanlog,sdlog,gam) {integrate(left, lower = 0, upper = 1, meanlog=meanlog, sdlog=sdlog, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(0,1,0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,meanlog,sdlog,gam) {qlnorm(x, meanlog,sdlog)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(meanlog,sdlog,gam) {integrate(right, lower = 0, upper = 1, meanlog=meanlog, sdlog=sdlog, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(0,1,0.5)

 
xlnorm <- function(gam, meanlog,sdlog)
{
       #(Extremiles: by Daouia.A, June1, 2015)
       #inputs:
       #        gam = extremile order in (0,1)
       #(meanlog,sdlog)  = mean and standard deviation of the distribution on the log scale with default values of 0 and 1 resp. 
       #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft(meanlog,sdlog,gam);}
       else {res = xiright(meanlog,sdlog,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xlnorm(gam, mean=0, sd=1)
     # alpha = plnorm(xlnorm(gam, mean=0, sd=1), mean=0, sd=1);  #\alpha(\gamma) = ...
     # qlnorm(alpha, mean=0, sd=1)




#_______________________________________________________________
#
#   (9.15) MC : Lognormal(0,0.5) :
#
#_______________________________________________________________
#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	#--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
   	alpha = plnorm(xlnorm(gam, meanlog=0, sdlog=1), meanlog=0, sdlog=1);       #Lognormal(meanlog,sdlog)
	
	#True extremile(gamma) = true quantile(alpha):
	tVaR = xlnorm(gam, meanlog=0, sdlog=1/2) * matrix(1,B,1);  #Lognormal(meanlog,sdlog=1/2)
	
	# #------------------------------------------------------------------------------------------------------

	# #\alpha(\gamma):
	# knp = 0; #floor((5/100) * n/floor(n^0.1));        #floor(0.10 * n); #floor(n^0.5);
# #	knp = floor((10/100) * n/floor(n^0.1));        #floor(0.10 * n); #floor(n^0.5);
	
	# kn = 0;
	
	# alpha = plnorm(xlnorm_trim(gam,0,1/2,kn,knp,n), 0,1/2);       #Lognormal(meanlog=0,sdlog=1/2)

	
	# #True extremile_{gamma}(kn,kn') = true quantile_{alpha}:
	# tVaR = xlnorm_trim(gam,0,1/2,kn,knp,n) * matrix(1,B,1);      #Lognormal(meanlog,sdlog)

	# #------------------------------------------------------------------------------------------------------
	


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rlnorm(n, meanlog=0, sdlog=1/2);       #Lognormal(meanlog,sdlog)
	#------------------------------------------------------------------------------------------------------
 
                  
                 #computation of the estimates:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_extremile(ytab,gam);                 
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 

# #-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//lognromal.RData")
# ##


#__________________________________
#
#  (9.16) True extremile of the Weibull distribution:  
#            
#__________________________________
 
xweibull <- function(gam, shape,scale)
{
       #(Extremiles: by Daouia.A, June1, 2015)
       #inputs:
       #        gam = extremile order in (0,1)
       #(shape,scale)  = shape and scale parameters, the latter defaulting to 1. 
       #output: theoretical (gam)th extremile $\xi_{gam}$   
       #__________________________________
       
      # xi(gamma) for gamma <= 0.5 :
     left<-function(x,shape,scale,gam) {qweibull(x, shape,scale)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(shape,scale,gam) {integrate(left, lower = 0, upper = 1, shape=shape,scale=scale, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(1,pi,0.5)
     #__________________________________             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,shape,scale,gam) {qweibull(x, shape,scale)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(shape,scale,gam) {integrate(right, lower = 0, upper = 1,shape=shape,scale=scale, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(1,pi,0.5)
     #__________________________________      
                     
       if (gam <= 0.5){res = xileft(shape,scale,gam);}
       else {res = xiright(shape,scale,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xweibull(gam, shape = 1, scale = pi)
     # alpha = pweibull(xweibull(gam, shape = 1, scale = pi), shape = 1, scale = pi);  #\alpha(\gamma) 
     # qweibull(alpha, shape = 1, scale = pi)


#_______________________________________________________________
#
#   (9.17) MC :  Weibull(1,pi) :
#
#_______________________________________________________________
#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	#--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
   	alpha = pweibull(xweibull(gam, shape = 1, scale = pi), shape = 1, scale = pi);       #Weibull(shape,scale)
	
	#True extremile(gamma) = true quantile(alpha):
	tVaR = xweibull(gam, shape = 1, scale = pi) * matrix(1,B,1);  #Weibull(shape,scale)
	
	#------------------------------------------------------------------------------------------------------


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rweibull(n, shape = 1, scale = pi);       #Weibull(shape,scale)
	#------------------------------------------------------------------------------------------------------
 
                  
                 #computation of the estimates:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_extremile(ytab,gam);                 
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 

# #-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//weibull.RData")
# ##


#_______________________________________________________________
#
#   (9.18)  Figure 1 in Supplement B.1 : MSE_ordinary.pdf
#
#_______________________________________________________________

 dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(4,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


#---------------------------------------------------------------------------------------------------
#  Normal(0,1) :     (source code : main_ordinary_gaussian)                         
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/nromal.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
 labs(title = "Normal"),
  vp=define_region(1,1)) 


#---------------------------------------------------------------------------------------------------
#  Exponentiel(1) :     (source code : main_ordinary_exp)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/exponentiel.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Exponential"),
  vp=define_region(1,2))  


#---------------------------------------------------------------------------------------------------
#  Gamma(3,1) :     (source code : main_ordinary_gamma)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/gamma.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Gamma"),
  vp=define_region(2,1))   
  

#---------------------------------------------------------------------------------------------------
#  Uniform[0,1] :     (source code : main_ordinary_uniform)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/uniform.RData")  
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Uniform"),
  vp=define_region(2,2))    
  
  
#---------------------------------------------------------------------------------------------------
#  Beta(2,2) :     (source code : main_ordinary_beta)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/beta.RData")  
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Beta"),
  vp=define_region(3,1)) 
  
  
#---------------------------------------------------------------------------------------------------
#  Chi-square(3) :     (source code : main_ordinary_chisq)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/Chisquare.RData")    
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Chi-square"),
  vp=define_region(3,2))  
  
  
#---------------------------------------------------------------------------------------------------
#  Lognormal(0,0.5) :     (source code : main_ordinary_lognormal)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/lognromal.RData")  
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Lognormal"),
  vp=define_region(4,1))    
  

#---------------------------------------------------------------------------------------------------
#  Weibull(1,pi) :     (source code : main_ordinary_weibull)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/weibull.RData")  
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Weibull"),
  vp=define_region(4,2))    
  
 


#_______________________________________________________________
#
#
#   (10)  Supplement B.2:  
#
#
#_______________________________________________________________
#__________________________________
#
#  (10.1) Empirical quantile: 
#
#__________________________________

empirical_quantile  <- function(ytab,gam)
{
      #inputs:
      #ytab = sample of nx1 obs
      # gam = quantile order in (0,1)
      #output: $gam$th sample quantile      
      
ysort=sort(ytab);
n=length(ytab);
       alphan=gam*n;
       if (floor(alphan) == alphan)
           {res=ysort[alphan];}
       else 
           {res=ysort[floor(alphan)+1];}
       return(res);
}



#__________________________________
#
#  (10.2) Empirical extremile: 
#
#__________________________________

sg <- function(gam)
{
	#input: gam = extremile order in (0,1)
      #output: power $s(gam)$ in the definition of the measure $K_{gam}$ [see daouia & Gijbels (Extremiles)]
      
return(log(1/2)/log(1-gam));
}

Kg <- function(gam,t)
{
	#input: gam = extremile order in (0,1)
	#           t = real number in the support [0,1] of the $K_{gam}$
      #output: measure $K_{gam}(t)$ [see daouia & Gijbels (Extremiles)]
res=((1-(1-t)^(sg(gam)))*(0<gam)*(gam<=0.5))+((t^(sg(1-gam)))*(0.5<gam)*(gam<1));     
return(res);
}


empirical_extremile <- function(ytab,gam)
{
	  #(Extremiles: by Daouia.A, May 4, 2015)
      #inputs:
      #ytab = sample of nx1 obs
      # gam = extremile order in (0,1)
      #output: $gam$th sample extremile          

sytab=sort(ytab);
res=0;
n=length(ytab);
for (i in 1:n)
    {res = res + ((Kg(gam,i/n)-Kg(gam,(i-1)/n))*sytab[i]);}
return(res);
}


#__________________________________
#
#  (10.3) Empirical trimmed extremile: 
#
#__________________________________

empirical_trimmed_extremile <- function(ytab,gam,kn,knp)
{
	  #(Extremiles: by Daouia.A, May 4, 2015)
      #inputs:
      #ytab = sample of nx1 obs
      # gam = extremile order in (0,1)
      # kn, knp : trimming numbers (e.g., kn=kn'= an intermediate sequence)
      #output: The $(k_n,k_n')$-trimmed sample $\gamma$th extremile $\widehat{\xi}_{\gamma}(k_n,k_n')$ 
      #        For  kn=kn'=0 we obtain the full empirical $\gamma$th extremile $\widehat{\xi}_{\gamma}$  
      # Recommandation: use in practice $kn=kn'= floor(n^epsilon)$ with $epsilon = 0.1, 0.2 or 0.3$.    

sytab=sort(ytab);
res=0;
n=length(ytab);
for (i in (kn+1):(n-knp))
    {res = res + ((Kg(gam,i/n)-Kg(gam,(i-1)/n))*sytab[i]);}
return(res);
}



#__________________________________
#
#  (10.4) True extremile $\xi_{\gamma}$
#      of the Burr distribution:  
#          
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,shape1, shape2, rate,gam) {qburr(x, shape1, shape2, rate)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(shape1, shape2, rate,gam) {integrate(left, lower = 0, upper = 1, shape1=shape1, shape2=shape2, rate=rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(2, 3, 1,0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,shape1, shape2, rate,gam) {qburr(x, shape1, shape2, rate)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(shape1, shape2, rate,gam) {integrate(right, lower = 0, upper = 1,  shape1=shape1, shape2=shape2, rate=rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(4,1,1,0.5)

 
xburr <- function(gam, shape1, shape2, rate)
{
       #(Extremiles: by Daouia.A, May 9, 2015)
       #inputs:
       #        gam   = extremile order in (0,1)
       #        shape1, shape2, rate = parameters. Must be strictly positive (see Actuar: Burr distribution). 
       #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft(shape1, shape2, rate,gam);}
       else {res = xiright(shape1, shape2, rate,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xburr(gam, 4,1,1)
     # alpha = pburr(xburr(gam, 4,1,1), 4,1,1);  #\alpha(\gamma) = ...
     # qburr(alpha, 4,1,1)
     # mburr(1, 3, 1, 1)
     # mburr(3, 4, 1, 1)
     # mburr(3, 2.5, 1.5, 1)
     

#__________________________________
#
#  (10.5) Population trimmed extremile $\xi_{\gamma}(kn,kn')$
#      of the Burr distribution:  
#          
#__________________________________

     #$\xi_{\gamma}(kn,kn')$ for gamma <= 0.5 :
     left<-function(x,shape1, shape2, rate,gam) {qburr(x, shape1, shape2, rate)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(shape1, shape2, rate,gam,kn,knp,n) {integrate(left, lower = (kn/n), upper = (1-(knp/n)), shape1=shape1, shape2=shape2, rate=rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(4,1,1,0.5,0,0,100)
             
        
     #$\xi_{\gamma}(kn,kn')$ for gamma >= 0.5 :
     right<-function(x,shape1, shape2, rate,gam) {qburr(x, shape1, shape2, rate)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(shape1, shape2, rate,gam,kn,knp,n) {integrate(right, lower = (kn/n), upper = (1-(knp/n)), shape1=shape1, shape2=shape2, rate=rate, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(4,1,1,0.5,0,0,100)

 
xburr_trim <- function(gam,shape1, shape2, rate,kn,knp,n)
{
       #(Extremiles: by Daouia.A, May 11, 2015)
       #inputs:
       #        gam = extremile order in (0,1)
       #        shape1, shape2, rate = parameters. Must be strictly positive (see Actuar: Burr distribution).
       #    kn, knp : trimming numbers (e.g., kn=kn'= an intermediate sequence)
       #          n = sample size
       #output: theoretical trimmed (gam)th extremile $\xi_{gam}(kn,kn')$   
              
       if (gam <= 0.5){res = xileft(shape1, shape2, rate,gam,kn,knp,n);}
       else {res = xiright(shape1, shape2, rate,gam,kn,knp,n);}
                 
      return(res);
}

     # ##Example [check that: Quantile_{alpha} = Extremile_{gam}(kn,knp)]
     # gam = 0.90; 
     # gam
     # n = 100;
     # kn= floor((5/100) * n/floor(n^0.1));
     # knp=kn;
     # xburr_trim(gam,4,1,1,kn,knp,n)
     # alpha = pburr(xburr_trim(gam,4,1,1,kn,knp,n), 4,1,1);  #\alpha(\gamma) = ...
     # alpha
     # qburr(alpha, 4,1,1)



#____________________________________________________
#
#   (10.6) Monte Carlo comparison: Burr(4,1) :
#
#____________________________________________________

 
#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
    	
    	
    #--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
	knp = floor((5/100) * n/floor(n^0.1));        #floor(0.10 * n); #floor(n^0.5);
#	knp = floor((10/100) * n/floor(n^0.1));        #floor(0.10 * n); #floor(n^0.5);
	
	#kn = knp;
	kn = 0;
	
	alpha = pburr(xburr_trim(gam,4,1,1,kn,knp,n), 4,1,1);       #Burr(4,1,1)

	
	#True extremile_{gamma}(kn,kn') = true quantile_{alpha}:
	tVaR = xburr_trim(gam,4,1,1,kn,knp,n) * matrix(1,B,1);      #Burr(4,1,1)
	
	#------------------------------------------------------------------------------------------------------

 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rburr(n, 4,1,1);       #Burr(4,1,1)
	#------------------------------------------------------------------------------------------------------
 
                  
                 # #computation of the estimates:
   	             # qu[b]  = empirical_quantile(ytab,alpha);
   	             # ext[b] = empirical_extremile(ytab,gam);
                  
                 #trimmed versions:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_trimmed_extremile(ytab,gam,kn,knp);
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 



   

#-----------------------------------------  Save data:  -------------------------------------------

# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//burr411_trimmed_5_onesided.RData")
##
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//burr411_trimmed_10_onesided.RData")


#__________________________________
#
#  (10.7) True extremile of the Pareto distribution:  
#          
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left<-function(x,shape, scale,gam) {qpareto(x, shape, scale)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(shape, scale,gam) {integrate(left, lower = 0, upper = 1, shape=shape, scale=scale, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(2, 2,0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right<-function(x,shape, scale,gam) {qpareto(x, shape, scale)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(shape, scale,gam) {integrate(right, lower = 0, upper = 1, shape=shape, scale=scale, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(2, 2,0.5)

 
xpareto <- function(gam, shape, scale)
{
       #(Extremiles: by Daouia.A, May 9, 2015)
       #inputs:
       #        gam   = extremile order in (0,1)
       #        shape, scale = parameters. Must be strictly positive. 
       #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft(shape, scale,gam);}
       else {res = xiright(shape, scale,gam);}
                 
      return(res);
}

     # ##Example [check that: Quantile(alpha) = Extremile(gam)]
     # gam = 0.99; 
     # xpareto(gam, 2, 2)
     # alpha = ppareto(xpareto(gam, 2, 2), 2, 2);  #\alpha(\gamma) = ...
     # qpareto(alpha, 2, 2)


#__________________________________
#
#  (10.8) Population trimmed extremile of the Pareto distribution:  
#          
#__________________________________

     #$\xi_{\gamma}(kn,kn')$ for gamma <= 0.5 :
     left<-function(x,shape,scale,gam) {qpareto(x, shape,scale)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft <-function(shape,scale,gam,kn,knp,n) {integrate(left, lower = (kn/n), upper = (1-(knp/n)), shape=shape ,scale=scale, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft(2,2,0.5,0,0,100)
             
        
     #$\xi_{\gamma}(kn,kn')$ for gamma >= 0.5 :
     right<-function(x,shape,scale,gam) {qpareto(x, shape,scale)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright <-function(shape,scale,gam,kn,knp,n) {integrate(right, lower = (kn/n), upper = (1-(knp/n)), shape=shape, scale=scale, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright(2,2,0.5,0,0,100)

 
xpareto_trim <- function(gam,shape,scale,kn,knp,n)
{
       #(Extremiles: by Daouia.A, May 9, 2015)
       #inputs:
       #        gam = extremile order in (0,1]
       #        shape, scale = parameters. Must be strictly positive.
       #    kn, knp : trimming numbers (e.g., kn=kn'= an intermediate sequence)
       #          n = sample size
       #output: theoretical trimmed (gam)th extremile $\xi_{gam}(kn,kn')$   
              
       if (gam <= 0.5){res = xileft(shape,scale,gam,kn,knp,n);}
       else {res = xiright(shape,scale,gam,kn,knp,n);}
                 
      return(res);
}

     # ##Example [check that: Quantile_{alpha} = Extremile_{gam}(kn,knp)]
     # gam = 0.90; 
     # gam
     # n = 100;
     # kn= floor((5/100) * n/floor(n^0.1));
     # knp=kn;
     # xpareto_trim(gam,2,2,kn,knp,n)
     # alpha = ppareto(xpareto_trim(gam,2,2,kn,knp,n), 2,2);  #\alpha(\gamma) = ...
     # alpha
     # qpareto(alpha, 2, 2)


#____________________________________________________
#
#   (10.9) Monte Carlo comparison: Pareto(4)
#
#____________________________________________________

#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    

#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 3000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
    	
    	
    #--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
    knp = floor((10/100) * n/floor(n^0.1));        #floor(0.10 * n); #floor(n^0.5);
	#kn = knp;
     kn = 0;
	alpha = ppareto(xpareto_trim(gam,4,1,kn,knp,n), 4,1);       #Pareto(4,1)
	
	#True extremile_{gamma}(kn,kn') = true quantile_{alpha}:
	tVaR = xpareto_trim(gam,4,1,kn,knp,n) * matrix(1,B,1);      #Pareto(4,1)
	
	#------------------------------------------------------------------------------------------------------

 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rpareto(n, 4,1);       #Pareto(4,1)
	#------------------------------------------------------------------------------------------------------
 
                  
                 # #computation of the estimates:
   	             # qu[b]  = empirical_quantile(ytab,alpha);
   	             # ext[b] = empirical_extremile(ytab,gam);
                  
                 #trimmed versions:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_trimmed_extremile(ytab,gam,kn,knp);
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 


#-----------------------------------------  Save data:  -------------------------------------------
#save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, #MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, #Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//#pareto2_2_trimmed.RData")
##
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//pareto41_trimmed.RData")
##
#save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//pareto41_trim_10_onesided.RData")


#__________________________________
#
#  (10.10) True extremile of the Student t(df) distribution:  
#          
#__________________________________

     # xi(gamma) for gamma <= 0.5 :
     left_student<-function(x,df,gam) {qt(x, df, ncp=0)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft_student <-function(df,gam) {integrate(left_student, lower = 0, upper = 1, df=df, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft_student(3,0.5)
             
        
     # xi(gamma) for gamma >= 0.5 :
     right_student<-function(x,df,gam) {qt(x, df, ncp = 0)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright_student <-function(df,gam) {integrate(right_student, lower = 0, upper = 1, df=df, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright_student(3,0.5)

 
xt <- function(gam, df)
{
       #(Extremiles: by Daouia.A, May 3, 2015)
       #inputs:
       #        gam = extremile order in (0,1]
       #        df  = Student's degree of freedom
       #output: theoretical (gam)th extremile $\xi_{gam}$   
              
       if (gam <= 0.5){res = xileft_student(df,gam);}
       else {res = xiright_student(df,gam);}
                 
      return(res);
}

     ##Example [check that: Quantile(alpha) = Extremile(gam)]
     #df=3;
     #gam = 0.99; 
     #xt(gam, df)
     #alpha = pt(xt(gam, df), df);  #\alpha(\gamma) = ...
     #qt(alpha, df)



#__________________________________
#
#  (10.11) Population trimmed extremile of the Student t(df) distribution:  
#          
#__________________________________

     #$\xi_{\gamma}(kn,kn')$ for gamma <= 0.5 :
     left_student<-function(x,df,gam) {qt(x, df, ncp=0)*(log(1/2)/log(1-gam))*(1-x)^((log(1/2)/log(1-gam))-1)}
     #
     xileft_student <-function(df,gam,kn,knp,n) {integrate(left_student, lower = (kn/n), upper = (1-(knp/n)), df=df, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xileft_student(3,0.5,0,0,100)
             
        
     #$\xi_{\gamma}(kn,kn')$ for gamma >= 0.5 :
     right_student<-function(x,df,gam) {qt(x, df, ncp = 0)*(log(1/2)/log(gam))*x^((log(1/2)/log(gam))-1)}
     #
     xiright_student <-function(df,gam,kn,knp,n) {integrate(right_student, lower = (kn/n), upper = (1-(knp/n)), df=df, gam=gam, rel.tol=5*10^8*.Machine$double.eps)$value }
     #
     #xiright_student(3,0.5,0,0,100)

 
xt_trim <- function(gam,df,kn,knp,n)
{
       #(Extremiles: by Daouia.A, May 4, 2015)
       #inputs:
       #        gam = extremile order in (0,1]
       #        df  = Student's degree of freedom
       #    kn, knp : trimming numbers (e.g., kn=kn'= an intermediate sequence)
       #          n = sample size
       #output: theoretical trimmed (gam)th extremile $\xi_{gam}(kn,kn')$   
              
       if (gam <= 0.5){res = xileft_student(df,gam,kn,knp,n);}
       else {res = xiright_student(df,gam,kn,knp,n);}
                 
      return(res);
}

     # ##Example [check that: Quantile_{alpha} = Extremile_{gam}(kn,knp)]
     # df=3;
     # gam = 0.90; 
     # gam
     # n = 100;
     # kn= floor((5/100) * n/floor(n^0.1));
     # knp=kn;
     # xt_trim(gam,df,kn,knp,n)
     # alpha = pt(xt_trim(gam,df,kn,knp,n), df);  #\alpha(\gamma) = ...
     # alpha
     # qt(alpha, df)



#____________________________________________________
#
#   (10.12) Monte Carlo comparison : Student t(3) 
#
#____________________________________________________

 
#values of gamma:
gamma_tab = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9); 
ngam = length(gamma_tab);  
 
 
#values of n:
n_tab = seq(100,3000,by=100);
nn = length(n_tab);    


#Matrix of MSE(xi_{gamma})/MSE(q_{alpha(gamma)}):
ratio_MSE = matrix(0,ngam,nn); #each line corresponds to a fixed "gamma"

#Matrix of Bias(xi_{gamma}) - Bias(q_{alpha(gamma)}):
difference_Bias =  matrix(0,ngam,nn); 
  
        
#Nb of simulated samples:   	
B = 2000;

   
for (i in 1:ngam)  
{
	#\gamma:
	gam = gamma_tab[i];  
	
	
	
	# #--------------------------------------Modify "distribution" here (1)----------------------------------
	# #\alpha(\gamma):
   	# alpha = pt(xt(gam, df=3), df=3);       #Student t(3)
	
	# #True extremile(gamma) = true quantile(alpha):
	# tVaR = xt(gam, df=3) * matrix(1,B,1);  #Student t(3)
	
	# #------------------------------------------------------------------------------------------------------


	
	for (j in 1:nn)  
    {
    	#sample size:
    	n = n_tab[j];
    	
    	
    #--------------------------------------Modify "distribution" here (1)----------------------------------
	#\alpha(\gamma):
	kn = floor((10/100) * n/floor(n^0.1));      
	#kn = floor((5/100) * n); 
	knp = kn;
	alpha = pt(xt_trim(gam,df=2,kn,knp,n), df=2);       #Student t(2)
	
	#True extremile_{gamma}(kn,kn') = true quantile_{alpha}:
	tVaR = xt_trim(gam,df=2,kn,knp,n) * matrix(1,B,1);  #Student t(2)
	
	#------------------------------------------------------------------------------------------------------

 
        #Initialization of the VaR estimators:
        qu  = matrix(0,B,1);
        ext = matrix(0,B,1);        
 
        #Monte-Carlo trials:
        for (b in 1:B)
        {         
        	
        	             
  	#--------------------------------------Modify "distribution" here (2)----------------------------------      	                
                 #simulated data:
                 ytab = rt(n, df=2);       #Student t(3)
	#------------------------------------------------------------------------------------------------------
 
                  
                 # #computation of the estimates:
   	             # qu[b]  = empirical_quantile(ytab,alpha);
   	             # ext[b] = empirical_extremile(ytab,gam);
                  
                 #trimmed versions:
   	             qu[b]  = empirical_quantile(ytab,alpha);
   	             ext[b] = empirical_trimmed_extremile(ytab,gam,kn,knp);
   	      }
    	    
        MSE_ext = colMeans((tVaR - ext)^2);
        MSE_qu  = colMeans((tVaR - qu)^2);        
        ratio_MSE[i,j] = MSE_ext/MSE_qu;
        
        Bias_ext = colMeans(ext - tVaR);
        Bias_qu  = colMeans(qu - tVaR);
        difference_Bias[i,j] = Bias_ext - Bias_qu;
      }	
} 
   
MSE_gamma01 = ratio_MSE[1,];
MSE_gamma02 = ratio_MSE[2,]; 
MSE_gamma03 = ratio_MSE[3,]; 
MSE_gamma04 = ratio_MSE[4,]; 
MSE_gamma05 = ratio_MSE[5,]; 
MSE_gamma06 = ratio_MSE[6,];
MSE_gamma07 = ratio_MSE[7,]; 
MSE_gamma08 = ratio_MSE[8,]; 
MSE_gamma09 = ratio_MSE[9,]; 
   

Bias_gamma01 = difference_Bias[1,];
Bias_gamma02 = difference_Bias[2,]; 
Bias_gamma03 = difference_Bias[3,]; 
Bias_gamma04 = difference_Bias[4,]; 
Bias_gamma05 = difference_Bias[5,]; 
Bias_gamma06 = difference_Bias[6,];
Bias_gamma07 = difference_Bias[7,]; 
Bias_gamma08 = difference_Bias[8,]; 
Bias_gamma09 = difference_Bias[9,]; 

#-----------------------------------------  Save data:  -------------------------------------------
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//student_t3_trimmed_10.RData")
##
# save(n_tab,  MSE_gamma01, MSE_gamma02, MSE_gamma03, MSE_gamma04, MSE_gamma05, MSE_gamma06, MSE_gamma07, MSE_gamma08, MSE_gamma09, Bias_gamma01, Bias_gamma02, Bias_gamma03, Bias_gamma04, Bias_gamma05, Bias_gamma06, Bias_gamma07, Bias_gamma08, Bias_gamma09, file="/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R//student_t3_trimmed.RData") # c'est: trimmed_5


#_______________________________________________________________
#
#
#   (10.13)  Figure 2 in Supplement B.2:  MSE_trimmed.pdf
#
#
#_______________________________________________________________

 dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(3,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


#---------------------------------------------------------------------------------------------------
#  Burr(4,1) 5% :     (source code : main_ordinary_burr)                         
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/burr411_trimmed_5_onesided.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Burr (5%)"),
  vp=define_region(1,1)) 


#---------------------------------------------------------------------------------------------------
#  Burr(4,1) 10% :     (source code : main_ordinary_burr)                         
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/burr411_trimmed_10_onesided.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Burr (10%)"),
  vp=define_region(1,2))  


#---------------------------------------------------------------------------------------------------
#  Pareto(4) 5% :     (source code : main_ordinary_pareto)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/pareto41_trimmed.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Pareto (5%)"),
  vp=define_region(2,1))   
  

#---------------------------------------------------------------------------------------------------
#  Pareto(4) 10% :     (source code : main_ordinary_pareto)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/pareto41_trim_10_onesided.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Pareto (10%)"),
  vp=define_region(2,2))    
  
  
  
#---------------------------------------------------------------------------------------------------
#  Student t(3)  5% :     (source code : main_ordinary_student)                              
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/student_t3_trimmed.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Student (5%)"),
  vp=define_region(3,1)) 
  
  
#---------------------------------------------------------------------------------------------------
#  Student t(3)  10% :     (source code : main_ordinary_student)                             
#---------------------------------------------------------------------------------------------------
load("/Users/daouia/Desktop/CICT_July_18/daouia_gijbels_papers/extremiles_codes_R/student_t3_trimmed_10.RData")
df_MSE <- data.frame(n_tab,  MSE_gamma01,  MSE_gamma03,  MSE_gamma05,  MSE_gamma07,  MSE_gamma09 )   
names(df_MSE) <- c("Sample_size_n",  "0.1", "0.3",  "0.5",  "0.7",  "0.9" )
ecm <- melt(df_MSE, id = "Sample_size_n")
dd <- qplot(Sample_size_n, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("0.1" = '#3399ff',"0.3" = 'red',"0.5" = '#9ACD32', "0.7" = 'orange',"0.9" = '#BA55D3' )) +
scale_y_continuous("Ratio") +
#scale_y_continuous(name = "root MSE", limits = c(0,0.5)) +
 scale_x_continuous("n")+
labs(title = "Student (10%)"),
  vp=define_region(3,2))  
  



#_______________________________________________________________
#
#
#   (11)  Supplement C:
#
#
#_______________________________________________________________
#__________________________________
#
#    (11.1) L-extremile: 
#
#__________________________________
sg <- function(gam)
{
	#input: gam = extremile order in (0,1)
      #output: power $s(gam)$ in the definition of the measure $K_{gam}$ [see daouia & Gijbels (Extremiles)]
      
return(log(1/2)/log(1-gam));
}

Kg <- function(gam,t)
{
	#input: gam = extremile order in (0,1)
	#           t = real number in the support [0,1] of the $K_{gam}$
      #output: measure $K_{gam}(t)$ [see daouia & Gijbels (Extremiles)]
res=((1-(1-t)^(sg(gam)))*(0<gam)*(gam<=0.5))+((t^(sg(1-gam)))*(0.5<gam)*(gam<1));     
return(res);
}

empirical_extremile <- function(ytab,gam)
{
	  #(Extremiles: by Daouia.A, May 4, 2015)
      #inputs:
      #ytab = sample of nx1 obs
      # gam = extremile order in (0,1)
      #output: $gam$th sample extremile          

sytab=sort(ytab);
res=0;
n=length(ytab);
for (i in 1:n)
    {res = res + ((Kg(gam,i/n)-Kg(gam,(i-1)/n))*sytab[i]);}
return(res);
}



#__________________________________
#
#    (11.2) LM-extremile: 
#
#__________________________________
Jg <- function(gam, t)
{
	#input: gam = extremile order in (0,1)
	#         t = real number in the support (0,1] of the density $J_{gam}=K'_{gam}$
  #output: weight-generating function $J_{gam}(t)$ [see daouia & Gijbels (Extremiles)]
  
  if ((0<gam) & (gam<=0.5)){res = (sg(gam))*(1-t)^(sg(gam)-1)}; 
  if ((0.5<gam) & (gam<1)) {res = (sg(1-gam))*t^(sg(1-gam)-1)};
  return(res)
}


LM_extremile <- function(ytab,gam)
{
      #inputs:
      #ytab = sample of nx1 obs
      # gam = extremile order in (0,1)
      #output: $gam$th als extremile $\widetilde{\xi}_{\gamma}$         

sytab=sort(ytab);
res=0;
n=length(ytab);
for (i in 1:n)
    {res = res + (Jg(gam,i/n)*sytab[i]);}
return(res/n);
}

    
#__________________________________
#
#    (11.3) M-extremile: 
#
#__________________________________    
 als_extremile <- function(ytab,probs)
{
       #inputs:
       #       ytab = sample of nx1 obs
       #        gam = order in (0,1)     
       #output: res = Ordinary $gam$th sample ALS estimator $\widetilde\xi_{gam}$.
       #$$ \widetilde{\xi}_{\gamma} = \frac{1}{n}  \sum_{i=1}^n  J_{gam}(i/n) * Ysort[i] $$
  #__________________________________
 sytab=sort(ytab)
 n=length(ytab)
    num=0;
    den=0;
    for (i in 1:n){
    num = num + (.Jg(probs,i/n)*sytab[i]);
    den = den + (.Jg(probs,i/n));    
    #print(.Jg(probs,i/n))
    }
    return(num/den)
}

  
    
#__________________________________
#
#    (11.4) PWM-extremile: 
#
#__________________________________

M10s <- function(ytab,s)
{
	#inputs: ytab = sample of nx1 obs   
	#        s = 1, 2,..., n-1
    #output: probability-weighted moment $\hat{M}_{1,0,s}$ [see daouia & Gijbels (Extremiles)]

    Sigma=0;
    n=length(ytab);
    ysort=sort(ytab);
    
    for (i in 1:(n-s))
        {
	     Pi = 1;
	     for (j in 1:s)
	        {Pi = Pi * ((n-i+1-j)/(n-j));}
         Sigma = Sigma + (Pi * ysort[i]);	
        }
return(Sigma/n);
}


M1r0 <- function(ytab,r)
{
	#inputs: ytab = sample of nx1 obs   
	#        r = 1, 2,..., n-1
    #output: probability-weighted moment $\hat{M}_{1,r,0}$ [see daouia & Gijbels (Extremiles)]

    Sigma=0;
    n=length(ytab);
    ysort=sort(ytab);
    
    for (i in (r+1):n)
        {
	     Pi = 1;
	     for (j in 1:r)
	        {Pi = Pi * ((i-j)/(n-j));}
         Sigma = Sigma + (Pi * ysort[i]);	
        }
return(Sigma/n);
}

pwm_extremile <- function(ytab,gam)
{
	  #(Extremiles: by Daouia.A, Feb 28, 2016)
      #inputs:
      #ytab = sample of nx1 obs
      # gam = extremile order in (0,1)
      #output: $gam$th PWM extremile $\tilde{\xi}_{\gamma}^{*}$         


if (gam <= 0.5) 
{
   res = sg(gam) * M10s(ytab,sg(gam)-1);
} 
else 
{
   res = sg(1-gam) * M1r0(ytab,sg(1-gam)-1); 
}

return(res)
}




#__________________________________
#
# (11.5) Load data:
#     Logreturns of the SP500 Index
#
#__________________________________

# library(tseries)
# start="1994-11-02"; end= "2009-12-31"
# aa = get.hist.quote(instrument = "^GSPC", start, end,
                # quote =  "Close",
                # provider = c("yahoo", "oanda"), method = NULL,
                           # retclass = c("zoo", "its", "ts"), quiet = 
# FALSE, drop = FALSE)
  # str(aa)

# #Logreturns of the SP500 Index:
# ab = diff(log(aa));


# # problem with API 
# https://stackoverflow.com/questions/44050518/is-yql-datatables-org-down-as-of-may-17-18-2017

# # New source:
# https://finance.yahoo.com/quote/%5EGSPC/history?period1=783730800&period2=1262214000&interval=1d&filter=history&frequency=1d


gspc <- read.csv2("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes/^GSPC.csv", sep=",", dec=".") 
aa.Date <- as.Date(as.character(gspc$Date))
aa <- zoo(gspc[,5], aa.Date)
ab = diff(log(aa))




#_______________________________________________________________
#
#
#   (11.6)  Figure 3 in Supplement C:  Logreturns of the SP500 Index
#
#
#_______________________________________________________________
#autoplot(as(aa,"ts"))
#autoplot(as(aa,"ts"), xlab = 'Year', ylab = 'Logreturns')
#autoplot(as(ab,"ts"), xlab = 'Year', ylab = 'Logreturns', ts.colour = 'blue')

autoplot(ab) +  geom_line(color='blue') + labs(x = "Time", y="Logreturns")



#__________________________________
#
#      (11.7) Calculations:
#
#__________________________________
ytab = as.numeric(ab);
#size of the global time series: 3818
N = length(ytab); 
#size of the rolling windows :  
  n = 500;
#Number of forecast cases :
  T = N-n;

                 # #Chosen value of gamma:
                 # #-----------------
                 # gam = 0.025; 
                 # sg(gam) # 27.37785
                 # #-----------------
                 # sgam = 27;                
                 # gam = 1 - exp(log(1/2)/sgam); # $\gamma = 0.02534539$
                 # sg(gam)



                 # #Chosen value of gamma:
                 # #-----------------
                 # gam = 0.01; 
                 # sg(gam) # 68.96756
                 # #-----------------
                 # sgam = 68;                
                 # gam = 1 - exp(log(1/2)/sgam); # $\gamma = 0.01014156$
                 # sg(gam)



                 # #-----------------
                 # gam = 0.008; 
                 # sg(gam) # 86.29636
                 # #-----------------
                 # sgam = 86;                
                 # gam = 1 - exp(log(1/2)/sgam); # $\gamma = 0.008027457$
                 # sg(gam)



                 # #-----------------
                 # gam = 0.006; 
                 # sg(gam) # 115.1776
                 # #-----------------
                 # sgam = 115;                
                 # gam = 1 - exp(log(1/2)/sgam); # $\gamma = 0.006009239$
                 # sg(gam)



                 # #-----------------
                 # gam = 0.004; 
                 # sg(gam) # 172.94
                 # #-----------------
                 # sgam = 172;                
                 # gam = 1 - exp(log(1/2)/sgam); # $\gamma = 0.004021816$
                 # sg(gam)




                 #-----------------
                 gam = 0.002; 
                 sg(gam) # 346.2269
                 #-----------------
                 sgam = 346;                
                 gam = 1 - exp(log(1/2)/sgam); # $\gamma = 0.00200131$
                 sg(gam)




                 #Realizing observations "y_t": 
                 y_realobs =  0 * c(1:T);
        
              
                 #Point forecasts "x^{(k)}_t":  
                 x_VM  =  0 * c(1:T); #k=1                         
                 x_M   =  0 * c(1:T); #k=2
                 x_L   =  0 * c(1:T); #k=3
                 x_pwm =  0 * c(1:T); #k=4
                 
                    
                
       for (t in 1:T)
               {                         
                 #Rolling window sample:
                 yytab = ytab[t:(n+t-1)]; 
                 
                 
                 #Realizing observation "y_t": 
                 y_realobs[t] = - ytab[n+t];
                 
                                      
                 #Vrai M-estimator (M):
                 x_VM[t]     = - als_extremile(yytab,gam);


                 #M-estimator (LM):
                 x_M[t]     = - LM_extremile(yytab,gam);
                 
                 
                 #L-estimator (L):
                 x_L[t]     = - empirical_extremile(yytab,gam);                 
                 
                                 
                 #PWM-estimator (PWM):
                 x_pwm[t]   = - pwm_extremile(yytab,gam);                                  
                  
                                
           } #fin de la boucle sur t=1:T
           
          
           ind             = order(y_realobs); 
           ysort_realobs   = y_realobs[ind];
           x_VM_concomit   = x_VM[ind];           
           x_M_concomit    = x_M[ind];
           x_L_concomit    = x_L[ind];         
           x_pwm_concomit  = x_pwm[ind];
           
           S_VM  = 0; 
           S_M   = 0;
           S_L   = 0;   
           S_pwm = 0; 
 for (t in 1:T)
         {            
           S_VM  = S_VM + (Jg(gam,t/T)*(x_VM_concomit[t] - ysort_realobs[t])^2);
           S_M   = S_M + (Jg(gam,t/T)*(x_M_concomit[t] - ysort_realobs[t])^2);
           S_L   = S_L + (Jg(gam,t/T)*(x_L_concomit[t] - ysort_realobs[t])^2);                     
           S_pwm = S_pwm + (Jg(gam,t/T)*(x_pwm_concomit[t] - ysort_realobs[t])^2); 
 
           } #fin de la boucle sur t=1:T 
           
         
# save(y_realobs, x_VM,  x_M,  x_L, x_pwm, file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast002.RData")          
#
# save(y_realobs, x_VM,  x_M,  x_L, x_pwm, file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast004.RData")          
#
# save(y_realobs, x_VM,  x_M,  x_L, x_pwm, file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast006.RData")          
#
# save(y_realobs, x_VM,  x_M,  x_L, x_pwm, file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast008.RData")          
#
# save(y_realobs, x_VM,  x_M,  x_L, x_pwm, file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast01.RData")          
#
# save(y_realobs, x_VM,  x_M,  x_L, x_pwm, file="/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast025.RData")          
              
  
 
#__________________________________
#
#      (11.8) Table 1 : \label{realized_loss}
#          The values of the realized loss for the 4 ordinary methods
#
#__________________________________
           
           S_M   = S_M/T;
           S_L   = S_L/T;
           S_VM  = S_VM/T;           
           S_pwm = S_pwm/T;
            
S_M
S_L
S_VM
S_pwm



#_______________________________________________________________
#
#
#   (11.9)  Figure 4 in Supplement C:  backtest.pdf
#
#
#_______________________________________________________________

  dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(3,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

#__________________________________
#   tau ~ 0.025
#__________________________________
load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast025.RData") 
df_Realizing <- data.frame(xxc = c(1:T), yyc = y_realobs)     
df_VM <- data.frame(xxc = c(1:T), yyc = x_VM)
df_M <- data.frame(xxc = c(1:T), yyc = x_M)
df_L <- data.frame(xxc = c(1:T), yyc = x_L)
df_pwm <- data.frame(xxc = c(1:T), yyc = x_pwm)
dd <- ggplot(data=df_Realizing, aes(xxc, yyc)) + geom_line(size = .3, colour = "grey", linetype=1)
print(dd+geom_line(size = .3, data = df_M, colour = "#3399ff", linetype=1) +
geom_line(size = .3, data = df_VM, colour = "black", linetype=1) +
geom_line(size = .3, data = df_L, colour = "red", linetype=1) +
geom_line(size = .3, data = df_pwm, colour = "green", linetype=1) +
scale_y_continuous("Backtest  extremile") + 
scale_x_continuous("t") +    
labs(title = "tau ~ 0.025")+
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))

#__________________________________
#   tau ~ 0.01
#__________________________________
load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast01.RData") 
df_Realizing <- data.frame(xxc = c(1:T), yyc = y_realobs)     
df_VM <- data.frame(xxc = c(1:T), yyc = x_VM)
df_M <- data.frame(xxc = c(1:T), yyc = x_M)
df_L <- data.frame(xxc = c(1:T), yyc = x_L)
df_pwm <- data.frame(xxc = c(1:T), yyc = x_pwm)
dd <- ggplot(data=df_Realizing, aes(xxc, yyc)) + geom_line(size = .3, colour = "grey", linetype=1)
print(dd+geom_line(size = .3, data = df_M, colour = "#3399ff", linetype=1) +
geom_line(size = .3, data = df_VM, colour = "black", linetype=1) +
geom_line(size = .3, data = df_L, colour = "red", linetype=1) +
geom_line(size = .3, data = df_pwm, colour = "green", linetype=1) +
scale_y_continuous("Backtest  extremile") + 
scale_x_continuous("t") +    
labs(title = "tau ~ 0.01")+
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,2))

#__________________________________
#   tau ~ 0.008
#__________________________________
load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast008.RData") 
df_Realizing <- data.frame(xxc = c(1:T), yyc = y_realobs)     
df_VM <- data.frame(xxc = c(1:T), yyc = x_VM)
df_M <- data.frame(xxc = c(1:T), yyc = x_M)
df_L <- data.frame(xxc = c(1:T), yyc = x_L)
df_pwm <- data.frame(xxc = c(1:T), yyc = x_pwm)
dd <- ggplot(data=df_Realizing, aes(xxc, yyc)) + geom_line(size = .3, colour = "grey", linetype=1)
print(dd+geom_line(size = .3, data = df_M, colour = "#3399ff", linetype=1) +
geom_line(size = .3, data = df_VM, colour = "black", linetype=1) +
geom_line(size = .3, data = df_L, colour = "red", linetype=1) +
geom_line(size = .3, data = df_pwm, colour = "green", linetype=1) +
scale_y_continuous("Backtest  extremile") + 
scale_x_continuous("t") +    
labs(title = "tau ~ 0.008")+
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,1))

#__________________________________
#   tau ~ 0.006
#__________________________________
load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast006.RData") 
df_Realizing <- data.frame(xxc = c(1:T), yyc = y_realobs)     
df_VM <- data.frame(xxc = c(1:T), yyc = x_VM)
df_M <- data.frame(xxc = c(1:T), yyc = x_M)
df_L <- data.frame(xxc = c(1:T), yyc = x_L)
df_pwm <- data.frame(xxc = c(1:T), yyc = x_pwm)
dd <- ggplot(data=df_Realizing, aes(xxc, yyc)) + geom_line(size = .3, colour = "grey", linetype=1)
print(dd+geom_line(size = .3, data = df_M, colour = "#3399ff", linetype=1) +
geom_line(size = .3, data = df_VM, colour = "black", linetype=1) +
geom_line(size = .3, data = df_L, colour = "red", linetype=1) +
geom_line(size = .3, data = df_pwm, colour = "green", linetype=1) +
scale_y_continuous("Backtest  extremile") + 
scale_x_continuous("t") +    
labs(title = "tau ~ 0.006")+
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,2))

#__________________________________
#   tau ~ 0.004
#__________________________________
load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast004.RData") 
df_Realizing <- data.frame(xxc = c(1:T), yyc = y_realobs)     
df_VM <- data.frame(xxc = c(1:T), yyc = x_VM)
df_M <- data.frame(xxc = c(1:T), yyc = x_M)
df_L <- data.frame(xxc = c(1:T), yyc = x_L)
df_pwm <- data.frame(xxc = c(1:T), yyc = x_pwm)
dd <- ggplot(data=df_Realizing, aes(xxc, yyc)) + geom_line(size = .3, colour = "grey", linetype=1)
print(dd+geom_line(size = .3, data = df_M, colour = "#3399ff", linetype=1) +
geom_line(size = .3, data = df_VM, colour = "black", linetype=1) +
geom_line(size = .3, data = df_L, colour = "red", linetype=1) +
geom_line(size = .3, data = df_pwm, colour = "green", linetype=1) +
scale_y_continuous("Backtest  extremile") + 
scale_x_continuous("t") +    
labs(title = "tau ~ 0.004")+
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(3,1))

#__________________________________
#   tau ~ 0.002
#__________________________________
load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/extremiles_codes//forecast002.RData") 
df_Realizing <- data.frame(xxc = c(1:T), yyc = y_realobs)     
df_VM <- data.frame(xxc = c(1:T), yyc = x_VM)
df_M <- data.frame(xxc = c(1:T), yyc = x_M)
df_L <- data.frame(xxc = c(1:T), yyc = x_L)
df_pwm <- data.frame(xxc = c(1:T), yyc = x_pwm)
dd <- ggplot(data=df_Realizing, aes(xxc, yyc)) + geom_line(size = .3, colour = "grey", linetype=1)
print(dd+geom_line(size = .3, data = df_M, colour = "#3399ff", linetype=1) +
geom_line(size = .3, data = df_VM, colour = "black", linetype=1) +
geom_line(size = .3, data = df_L, colour = "red", linetype=1) +
geom_line(size = .3, data = df_pwm, colour = "green", linetype=1) +
scale_y_continuous("Backtest  extremile") + 
scale_x_continuous("t") +    
labs(title = "tau ~ 0.002")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(3,2))







